@@ -5,23 +5,64 @@ standardize.treatments <- function(treatments) {
     treatments[order(treatments$id), ]
 }
 
-standardize.data <- function(data, treatment.levels) {
-    data$study <- as.factor(data$study)
+standardize.data <- function(data, treatment.levels, re.order=FALSE) {
+    data$study <- factor(as.factor(data$study))
     data$treatment <- factor(as.character(data$treatment), levels=treatment.levels)
-    data <- data[order(data$study, data$treatment), ]
-    rownames(data) <- seq(1:nrow(data))
+	if (re.order) {
+		na <- sapply(data$study, function(study) { sum(data$study == study) })
+		bl <- !is.na(data$diff)
+		data <- data[order(na, data$study, bl, data$treatment), ]
+	} else {
+		data <- data[order(data$study, data$treatment), ]
+	}
+	if (nrow(data) > 0) {
+		rownames(data) <- seq(1:nrow(data))
+	}
     data
 }
 
-mtc.network <- function(data, treatments=NULL, description="Network") {
+remove.onearm <- function(data, warn=FALSE) {
+    # Remove 1-arm studies
+    sel <- as.logical(sapply(data[,'study'], function(study) {
+        sum(data[,'study'] == study) > 1
+    }))
+
+	if (warn && !all(sel)) {
+		warning(paste("Removing", sum(!sel), "one-arm studies"))
+	}
+    data[sel, ]
+}
+
+mtc.network <- function(data=NULL, treatments=NULL, description="Network", data.re=NULL) {
+	if (is.null(data) && is.null(data.re)) {
+		stop("Either `data' or `data.re' (or both) must be specified")
+	}
     # standardize the data
-    if (!is.data.frame(data)) { 
-        data <- do.call(rbind, lapply(data, as.data.frame))
+    if (!is.null(data)) {
+		if (!is.data.frame(data)) {
+			data <- do.call(rbind, lapply(data, as.data.frame))
+		}
+		data <- remove.onearm(data, warn=TRUE)
+		mtc.validate.data.ab(data)
+    }
+    if (!is.null(data.re)) {
+		if (!is.data.frame(data.re)) {
+			data.re <- do.call(rbind, lapply(data.re, as.data.frame))
+		}
+		data.re <- remove.onearm(data.re, warn=TRUE)
+		mtc.validate.data.re(data.re)
     }
 
     # standardize the treatments
     if (is.null(treatments)) {
-        treatments <- unique(data$treatment)
+		data.treatments <- vector(mode="character")
+		if (!is.null(data)) {
+			data.treatments <- c(data.treatments, as.character(data$treatment))
+		}
+		if (!is.null(data.re)) {
+			data.treatments <- c(data.treatments, as.character(data.re$treatment))
+		}
+        treatments <- unique(data.treatments)
     }
     if (is.list(treatments) && !is.data.frame(treatments)) { 
         treatments <- as.data.frame(do.call(rbind, treatments))
@@ -33,11 +74,16 @@ mtc.network <- function(data, treatments=NULL, description="Network") {
 
     network <- list(
         description=description,
-        treatments=treatments,
-        data=standardize.data(data, levels(treatments$id))
-    )
+        treatments=treatments)
 
-    mtc.network.validate(network)
+	if (!is.null(data)) {
+		network <- c(network, list(data=standardize.data(data, levels(treatments$id))))
+	}
+	if (!is.null(data.re)) {
+		network <- c(network, list(data.re=standardize.data(data.re, levels(treatments$id), re.order=TRUE)))
+	}
+
+	mtc.network.validate(network)
 
     class(network) <- "mtc.network"
     network
@@ -94,9 +140,9 @@ read.mtc.network <- function(file) {
 write.mtc.network <- function(network, file) {
     root <- XML::newXMLNode("network")
     XML::xmlAttrs(root)["description"] <- network$description
-    type <- if ('responders' %in% colnames(network$data)) {
+    type <- if ('responders' %in% colnames(network[['data']])) {
         'rate'
-    } else if ('mean' %in% colnames(network$data)) {
+    } else if ('mean' %in% colnames(network[['data']])) {
         'continuous'
     } else {
         'none'
@@ -111,13 +157,13 @@ write.mtc.network <- function(network, file) {
     })
 
     studies <- XML::newXMLNode("studies", parent = root)
-    study <- sapply(levels(network$data$study), function(sid) {
+    study <- sapply(levels(network[['data']]$study), function(sid) {
         node <- XML::newXMLNode("study", parent = studies)
         XML::xmlAttrs(node)["id"] <- sid
         node
     })
 
-    apply(network$data, 1, function(row) {
+    apply(network[['data']], 1, function(row) {
         node <- XML::newXMLNode('measurement', parent=study[[row['study']]])
         XML::xmlAttrs(node)['treatment'] <- row['treatment']
         if (identical(type, 'rate')) {
@@ -134,14 +180,57 @@ write.mtc.network <- function(network, file) {
     cat(XML::saveXML(root), file=file)
 }
 
+mtc.validate.data.ab <- function(data) {
+	columns <- colnames(data)
+	contColumns <- c('mean', 'std.dev', 'sampleSize')
+	dichColumns <- c('responders', 'sampleSize')
+
+	if (contColumns[1] %in% columns && dichColumns[1] %in% columns) {
+		stop('Ambiguous whether data is continuous or dichotomous: both "mean" and "responders" present.')
+	}
+
+	if (contColumns[1] %in% columns && !all(contColumns %in% columns)) {
+		stop(paste('Continuous data must contain columns:', paste(contColumns, collapse=', ')))
+	}
+
+	if (dichColumns[1] %in% columns && !all(dichColumns %in% columns)) {
+		stop(paste('Dichotomous data must contain columns:', paste(dichColumns, collapse=', ')))
+	}
+}
+
+mtc.validate.data.re <- function(data) {
+	columns <- colnames(data)
+	reColumns <- c('diff', 'std.err')
+	if (!all(reColumns %in% columns)) {
+		stop(paste('data.re must contain columns: ', paste(reColumns, collapse=', ')))
+	}
+
+	baselineCount <- sapply(unique(data$study), function(study) { sum(is.na(data$diff[data$study == study])) })
+	if (!all(baselineCount == 1)) {
+		stop('Each study in data.re must have a unique baseline arm (diff=NA)')
+	}
+	if (!all(!is.na(data$std.err[!is.na(data$diff)]))) {
+		stop('All non-baseline arms in data.re must have std.err specified')
+	}
+
+	studies <- unique(data$study)
+	ma <- sapply(studies, function(study) { sum(data$study == study) > 2 })
+	ma <- studies[ma]
+	if (!all(!is.na(data$std.err[data$study %in% ma]))) {
+		stop('All multi-arm trials (> 2 arms) must have the std.err of the baseline specified')
+	}
+}
+
 mtc.network.validate <- function(network) { 
     # Check that there is some data
     stopifnot(nrow(network$treatments) > 0)  
-    stopifnot(nrow(network$data) > 0)
+    stopifnot(nrow(network[['data']]) > 0 || nrow(network[['data.re']]) > 0)
 
     # Check that the treatments are correctly cross-referenced and have valid names
-    stopifnot(all(network$data$treatment %in% network$treatments$id))
-    stopifnot(all(network$treatments$id %in% network$data$treatment))
+	all.treatments <- c(network[['data']]$treatment, network[['data.re']]$treatment)
+	all.treatments <- factor(all.treatments, levels=1:nlevels(network$treatments$id), labels=levels(network$treatments$id))
+    stopifnot(all(all.treatments %in% network$treatments$id))
+    stopifnot(all(network$treatments$id %in% all.treatments))
     idok <- regexpr("^[A-Za-z0-9_]+$", network$treatment$id) != -1
     if(!all(idok)) {
         stop(paste('Treatment name "',
@@ -149,22 +238,23 @@ mtc.network.validate <- function(network) {
             ' Treatment names may only contain letters, digits, and underscore (_).'), sep='')
     }
 
+	# Check that studies are not duplicated between $data and $data.re
+	if (!is.null(network[['data']]) && !is.null(network[['data.re']])) {
+		dup.study <- intersect(unique(network[['data']]$study), unique(network[['data.re']]$study))
+		if (length(dup.study) > 0) {
+			stop(paste('Studies', paste(dup.study, collapse=", "), 'occur in both data and data.re'))
+		}
+	}
+
     # Check that the data frame has a sensible combination of columns
-    columns <- colnames(network$data)
-    contColumns <- c('mean', 'std.dev', 'sampleSize')
-    dichColumns <- c('responders', 'sampleSize')
+	if (!is.null(network[['data']])) {
+		mtc.validate.data.ab(network[['data']])
+	}
 
-    if (contColumns[1] %in% columns && dichColumns[1] %in% columns) {
-        stop('Ambiguous whether data is continuous or dichotomous: both "mean" and "responders" present.')
-    }
-
-    if (contColumns[1] %in% columns && !all(contColumns %in% columns)) {
-        stop(paste('Continuous data must contain columns:', paste(contColumns, collapse=', ')))
-    }
-
-    if (dichColumns[1] %in% columns && !all(dichColumns %in% columns)) {
-        stop(paste('Dichotomous data must contain columns:', paste(dichColumns, collapse=', ')))
-    }
+	# Check data.re is well formed
+	if (!is.null(network[['data.re']])) {
+		mtc.validate.data.re(network[['data.re']])
+	}
 }
 
 as.treatment.factor <- function(x, network) {
@@ -177,15 +267,49 @@ as.treatment.factor <- function(x, network) {
     }
 }
 
+mtc.merge.data <- function(network) {
+	data.frame(
+		study=c(
+			as.character(network[['data']]$study),
+			as.character(network[['data.re']]$study)),
+		treatment=as.treatment.factor(c(
+			network[['data']]$treatment,
+			network[['data.re']]$treatment), network),
+		stingsAsFactors=FALSE)
+}
+
+mtc.studies.list <- function(network) {
+	rle(as.character(mtc.merge.data(network)$study))
+}
+
 mtc.study.design <- function(network, study) {
-    data <- network$data
-    sort(data$treatment[data$study == study])
+    data <- mtc.merge.data(network)
+    data$treatment[data$study == study]
 }
 
 coerce.factor <- function(x, prototype) {
     factor(x, levels=1:nlevels(prototype), labels=levels(prototype))
 }
 
+# See nodesplit-auto draft for definition
+has.indirect.evidence <- function(network, t1, t2) {
+	has.both <- sapply(mtc.studies.list(network)$values, function(study) {
+		all(c(t1, t2) %in% mtc.study.design(network, study))
+	})
+
+	data <- rbind(network[['data']], network[['data.re']])
+	data <- data[!has.both[data[['study']]] | (data[['treatment']] != t1 & data[['treatment']] != t2), ]
+	data <- remove.onearm(data)
+
+	if (nrow(data) > 0) {
+		n <- mtc.network(data)
+		g <- mtc.network.graph(n)
+		all(c(t1, t2) %in% V(g)$name) && as.logical(is.finite(shortest.paths(as.undirected(g), t1, t2)))
+	} else {
+		FALSE
+	}
+}
+
 mtc.treatment.pairs <- function(treatments) {
     n <- length(treatments)
     t1 <- do.call(c, lapply(1:(n-1), function(i) { rep(treatments[i], n - i) }))
@@ -196,7 +320,7 @@ mtc.treatment.pairs <- function(treatments) {
 # Get all comparisons with direct evidence from the data set.
 # Returns a (sorted) data frame with two columns (t1 and t2).
 mtc.comparisons <- function(network) {
-    data <- network$data
+    data <- mtc.merge.data(network)
 
     # Identify the unique "designs" (treatment combinations)
     design <- function(study) { mtc.study.design(network, study) }
@@ -235,17 +359,25 @@ mtc.network.graph <- function(network) {
 ## mtc.network class methods
 print.mtc.network <- function(x, ...) {
     cat("MTC dataset: ", x$description, "\n", sep="")
-    print(x$data)
+	if (!is.null(x[['data']])) {
+		cat('Arm-level data: \n')
+		print(x[['data']])
+	}
+	if (!is.null(x[['data.re']])) {
+		cat('Relative effect data: \n')
+		print(x[['data.re']])
+	}
 }
 
 summary.mtc.network <- function(object, ...) {
-    studies <- levels(object$data[,1])
-    m <- sapply(object$treatments[,1], function(treatment) {
+	data <- mtc.merge.data(object)
+    studies <- levels(data$study)
+    m <- sapply(object$treatments$id, function(treatment) {
         sapply(studies, function(study) { 
-            any(object$data[,1] == study & object$data[,2] == treatment)
+            any(data$study == study & data$treatment == treatment)
         })
     })
-    colnames(m) <- object$treatments[,1]
+    colnames(m) <- object$treatments$id
     x <- as.factor(apply(m, 1, sum))
     levels(x) <- sapply(levels(x), function(y) { paste(y, "arm", sep="-") })
     list("Description"=paste("MTC dataset: ", object$description, sep=""),
