@@ -37,7 +37,8 @@ tree.relative.effect <- function(g, t1, t2) {
 # parameters and generate a two-column matrix in which each row is a comparison
 extract.comparisons <- function(parameters) {
     x <- parameters[grep("^d\\.", parameters)]
-    matrix(unlist(strsplit(x, "\\.")), ncol=3, byrow=TRUE)[,-1]
+    x <- matrix(unlist(strsplit(x, "\\.")), ncol=3, byrow=TRUE)[,-1]
+    matrix(x, ncol=2) # seriously, R?
 }
 
 spanning.tree.mtc.result <- function(result) {
@@ -52,7 +53,7 @@ spanning.tree.mtc.result <- function(result) {
 }
 
 relative.effect <- function(result, t1, t2 = c(), preserve.extra=TRUE) {
-    if(result$model$type != "Consistency") stop("Cannot apply relative.effect to this model")
+    if(tolower(result$model$type) != 'consistency') stop("Cannot apply relative.effect to this model")
 
     # Build relative effect transformation matrix
     network <- result$model$network
